# coding: utf-8

__author__ = "Ondrej Jurcak"

import logging
from configparser import ConfigParser

from aws_xray_sdk import global_sdk_config
from aws_xray_sdk.core.async_context import AsyncContext
from aws_xray_sdk.core import xray_recorder, patch_all, patch
from aws_xray_sdk.core.utils import stacktrace
from aws_xray_sdk.ext.flask.middleware import XRayMiddleware
from aws_xray_sdk.ext.flask_sqlalchemy.query import XRayFlaskSqlAlchemy
from aws_xray_sdk.ext.util import construct_xray_header, inject_trace_header
from sentry_sdk.integrations.logging import ignore_logger

try:
    from celery import signals

    CELERY_INSTALLED = True
except ImportError:
    CELERY_INSTALLED = False


class XRaySettingsConfigParser:
    def __init__(self, settings):
        if "XRAY" in settings:
            xray = settings["XRAY"]
            self.enabled = xray.getboolean("ENABLED", False)
            self.service_name = xray.get("SERVICE_NAME")
            self.plugins = xray.get("PLUGINS", "'ECSPlugin', 'EC2Plugin'")
            self.sampling = xray.getboolean("SAMPLING", False)
            self.stream_sql = xray.getboolean("STREAM_SQL", True)
            self.daemon = xray.get("DAEMON", "xray-daemon:2000")
            self.patch_modules = xray.get("PATCH_MODULES", None)
            self.logging_enabled = xray.getboolean("LOGGING_ENABLED", False)
            self.sqlalchemy = xray.getboolean("SQLALCHEMY", False)
            self.middleware = xray.getboolean("MIDDLEWARE", True)
            self.sentry_logging = xray.getboolean("SENTRY_LOGGING", False)
            self.async_context = xray.getboolean("ASYNC_CONTEXT", False)
        else:
            self.enabled = False
            self.service_name = None
            self.plugins = "'ECSPlugin', 'EC2Plugin'"
            self.sampling = False
            self.stream_sql = True
            self.daemon = "xray-daemon:2000"
            self.patch_modules = None
            self.logging_enabled = False
            self.sqlalchemy = False
            self.middleware = True
            self.sentry_logging = False
            self.async_context = False


class XRaySettingsLazySettings:
    def __init__(self, settings):
        self.enabled = settings.XRAY_ENABLED
        self.service_name = settings.XRAY_SERVICE_NAME
        self.plugins = settings.XRAY_PLUGINS
        self.sampling = settings.XRAY_SAMPLING
        self.stream_sql = settings.XRAY_STREAM_SQL
        self.daemon = settings.XRAY_DAEMON
        self.patch_modules = settings.XRAY_PATCH_MODULES
        self.logging_enabled = settings.XRAY_LOGGING_ENABLED
        self.sqlalchemy = settings.XRAY_SQLALCHEMY
        self.middleware = settings.XRAY_MIDDLEWARE
        self.sentry_logging = settings.XRAY_SENTRY_LOGGING
        self.async_context = settings.XRAY_ASYNC_CONTEXT


class XRayConfigurator:
    def __init__(self, settings, application=None) -> None:
        logging.getLogger("xray_configurator_logger")
        if isinstance(settings, ConfigParser):
            self.settings = XRaySettingsConfigParser(settings)
        else:
            self.settings = XRaySettingsLazySettings(settings)

        if not self.settings.enabled:
            logging.info("xray sdk disabled")
            global_sdk_config.set_sdk_enabled(False)
            return

        self._init_xray(self.settings)

        if application:
            if self.settings.sqlalchemy:
                logging.info("XRayFlaskSqlAlchemy enabled")
                XRayFlaskSqlAlchemy(application)
            if self.settings.middleware:
                XRayMiddleware(application, xray_recorder)

    def _init_xray(self, settings):
        logging.info("xray sdk enabled " + str(settings.enabled))
        global_sdk_config.set_sdk_enabled(True)
        xray_recorder.configure(
            service=settings.service_name,
            plugins=eval(settings.plugins),
            sampling=settings.sampling,
            stream_sql=settings.stream_sql,
            daemon_address=settings.daemon,
            context_missing="LOG_ERROR",
            context=AsyncContext() if settings.async_context else None,
        )
        if not settings.patch_modules:
            patch_all()
        else:
            patch(eval(settings.patch_modules))

        if settings.logging_enabled:
            logging.info("XRay logging enabled")
            logging.getLogger("aws_xray_sdk").setLevel(logging.DEBUG)
        else:
            logging.info("XRay logging disabled")
            logging.getLogger("aws_xray_sdk").setLevel(logging.NOTSET)

        if not settings.sentry_logging:
            ignore_logger("aws_xray_sdk")
            ignore_logger("aws_xray_sdk.core.emitters.udp_emitter")
            ignore_logger("aws_xray_sdk.core.sampling.rule_poller")
            ignore_logger("aws_xray_sdk.core.sampling.target_poller")


if CELERY_INSTALLED:

    @signals.task_prerun.connect
    def task_prerun(task_id, task, *args, **kwargs):
        logging.info("task prerun task_id: " + task_id)
        xray_header = construct_xray_header(task.request)
        logging.info(
            "xray header constructed. Root: {}, Parent: {}".format(
                xray_header.root, xray_header.parent
            )
        )
        segment = xray_recorder.begin_segment(
            name=task.name,
            traceid=xray_header.root,
            parent_id=xray_header.parent,
        )
        segment.save_origin_trace_header(xray_header)
        segment.put_metadata("task_id", task_id, namespace="celery")

    @signals.task_postrun.connect()
    def task_postrun(task_id, *args, **kwargs):
        logging.info("task post run task_id: " + task_id)
        xray_recorder.end_segment()

    @signals.before_task_publish.connect
    def before_task_publish(sender, headers, **kwargs):
        logging.info("before taskpublish sender: " + sender)
        subsegment = xray_recorder.begin_subsegment(
            name=sender,
            namespace="remote",
        )

        if subsegment is None:
            logging.info("task taskpublish not in segment")
            return
        subsegment.put_metadata("task_id", headers.get("id"), namespace="celery")
        inject_trace_header(headers, subsegment)

    @signals.after_task_publish.connect
    def xray_after_task_publish(**kwargs):
        logging.info("after task publish")
        xray_recorder.end_subsegment()

    @signals.task_failure.connect
    def xray_task_failure(einfo, **kwargs):
        logging.info("task_failure")
        segment = xray_recorder.current_segment()
        if segment is None:
            logging.info("task failure not in segment")
            return
        if einfo:
            stack = stacktrace.get_stacktrace(limit=xray_recorder.max_trace_back)
            try:
                segment.add_exception(einfo.exception, stack)
            except FileNotFoundError as e:  # os.getcwd() crashes sometimes we need to handle this issue
                logging.info("File not found error whne segment.add_exception")



import logging
import unittest
from unittest import TestCase

from unittest.mock import Mock, patch, call

from aws_xray_sdk import global_sdk_config
from aws_xray_sdk.core import xray_recorder

from libs.xray.xray_configurator.xray_configurator import XRayConfigurator


class ConfigTestCase(TestCase):
    @patch("libs.xray.xray_configurator.xray_configurator.patch_all")
    @patch("libs.xray.xray_configurator.xray_configurator.XRayMiddleware")
    @patch("libs.xray.xray_configurator.xray_configurator.XRayFlaskSqlAlchemy")
    def test_no_xray_section(self, mock_alchemy, mock_middleware, mock_patch_all):
        class ConfigMock:
            XRAY_ENABLED = False
            XRAY_SERVICE_NAME = "test-service-name"
            XRAY_PLUGINS = "test"
            XRAY_SAMPLING = True
            XRAY_STREAM_SQL = True
            XRAY_DAEMON = "daemon:200"
            XRAY_PATCH_MODULES = None
            XRAY_LOGGING_ENABLED = False
            XRAY_SQLALCHEMY = False
            XRAY_MIDDLEWARE = False
            XRAY_SENTRY_LOGGING = False

        XRayConfigurator(ConfigMock(), Mock())
        self.assertFalse(global_sdk_config.sdk_enabled())
        mock_alchemy.assert_not_called()
        mock_middleware.assert_not_called()
        mock_patch_all.assert_not_called()

    @patch("libs.xray.xray_configurator.xray_configurator.xray_recorder.configure")
    @patch("libs.xray.xray_configurator.xray_configurator.patch")
    @patch("libs.xray.xray_configurator.xray_configurator.XRayMiddleware")
    @patch("libs.xray.xray_configurator.xray_configurator.XRayFlaskSqlAlchemy")
    @patch("libs.xray.xray_configurator.xray_configurator.ignore_logger")
    def test_xray(self, mock_ignore_logger, mock_alchemy, mock_middleware, mock_patch, mock_configure):
        app_mock = Mock()
        class ConfigMock:
            XRAY_ENABLED = True
            XRAY_SERVICE_NAME = 'test-service-name'
            XRAY_DAEMON = '129.0.0.0:5000'
            XRAY_PLUGINS = "'ECSPlugin', 'EC2Plugin', 'ElasticBeanstalkPlugin'"
            XRAY_SAMPLING = True
            XRAY_STREAM_SQL = True
            XRAY_SQLALCHEMY = True
            XRAY_LOGGING_ENABLED = True
            XRAY_PATCH_MODULES = "'boto3','requests'"
            XRAY_MIDDLEWARE = True
            XRAY_SENTRY_LOGGING = True

        XRayConfigurator(ConfigMock(), app_mock)
        self.assertTrue(global_sdk_config.sdk_enabled())
        mock_configure.assert_called_once_with(context_missing='LOG_ERROR', daemon_address='129.0.0.0:5000', plugins=('ECSPlugin', 'EC2Plugin', 'ElasticBeanstalkPlugin'), sampling=True, service='test-service-name', stream_sql=True)
        self.assertEqual(logging.getLogger("aws_xray_sdk").level, logging.DEBUG)

        mock_alchemy.assert_called_once_with(app_mock)
        mock_middleware.assert_called_once_with(app_mock, xray_recorder)
        mock_patch.assert_called_once_with(('boto3', 'requests'))
        mock_ignore_logger.assert_not_called()

    @patch("libs.xray.xray_configurator.xray_configurator.xray_recorder.configure")
    @patch("libs.xray.xray_configurator.xray_configurator.patch_all")
    @patch("libs.xray.xray_configurator.xray_configurator.XRayMiddleware")
    @patch("libs.xray.xray_configurator.xray_configurator.XRayFlaskSqlAlchemy")
    @patch("libs.xray.xray_configurator.xray_configurator.ignore_logger")
    def test_xray_extensions_disabled(self, mock_ignore_logger, mock_alchemy, mock_middleware, mock_patch, mock_configure):
        app_mock = Mock()

        class ConfigMock:
            XRAY_ENABLED = True
            XRAY_SERVICE_NAME = 'test-service-name'
            XRAY_DAEMON = None
            XRAY_PLUGINS = "'ECSPlugin', 'EC2Plugin'"
            XRAY_SAMPLING = False
            XRAY_STREAM_SQL = False
            XRAY_SQLALCHEMY = False
            XRAY_LOGGING_ENABLED = False
            XRAY_PATCH_MODULES = None
            XRAY_MIDDLEWARE = False
            XRAY_SENTRY_LOGGING = False

        XRayConfigurator(ConfigMock(), app_mock)
        self.assertTrue(global_sdk_config.sdk_enabled())
        mock_configure.assert_called_once_with(context_missing='LOG_ERROR', daemon_address=None,
                                               plugins=('ECSPlugin', 'EC2Plugin'),
                                               sampling=False, service='test-service-name', stream_sql=False)
        self.assertEqual(logging.getLogger("aws_xray_sdk").level, logging.NOTSET)

        mock_alchemy.assert_not_called()
        mock_middleware.assert_not_called()
        mock_patch.assert_called_once_with()
        mock_ignore_logger.assert_has_calls([call('aws_xray_sdk'), call('aws_xray_sdk.core.emitters.udp_emitter'), call('aws_xray_sdk.core.sampling.rule_poller'), call('aws_xray_sdk.core.sampling.target_poller')])


if __name__ == "__main__":
    unittest.main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="xrayconfigurator",  # Replace with your own username
    version="0.0.32",
    author="Ondrej Jurcak",
    author_email="ondrej@surglogs.com",
    description="Configuration helper for xray",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=["aws-xray-sdk>=2.9.0", "sentry-sdk[flask]>=1.5.5"],
    setup_requires=["wheel"],
    test_suite="nose.collector",
    tests_require=["nose", "wheel", "flask-sqlalchemy>=2.4.4"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)
